#[cfg(feature = "server")]
use rusqlite::{types::ToSqlOutput, ToSql};
use serde::{Serialize, Deserialize};

#[derive(Clone, Copy, Debug, Serialize, Deserialize)]
pub enum Store {
	Aldi,
	Edeka,
	Dm,
	Lidl,
	Rewe,
	Tegut,
}

#[derive(Clone, Copy, Debug, Serialize, Deserialize)]
pub enum Account {
	Sumpf,
	Heinersyndikat,
}

#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct Inventory {
	pub acc: Account,
	pub cash: i64,
	pub vouchers: Vec<VoucherInventory>,
}

#[derive(Clone, Copy, Debug, Serialize, Deserialize)]
pub struct VoucherInventory {
	pub store: Store,
	pub count: i64,
}

impl TryFrom<&str> for Store {
	type Error = ();

	fn try_from(s: &str) -> Result<Self, Self::Error> {
		match s {
			"aldi" => Ok(Store::Aldi),
			"edeka" => Ok(Store::Edeka),
			"dm" => Ok(Store::Dm),
			"lidl" => Ok(Store::Lidl),
			"rewe" => Ok(Store::Rewe),
			"tegut" => Ok(Store::Tegut),
			_ => Err(()),
		}
	}
}

impl TryFrom<&str> for Account {
	type Error = ();

	fn try_from(s: &str) -> Result<Self, Self::Error> {
		match s {
			"sumpf" => Ok(Account::Sumpf),
			"hs" => Ok(Account::Heinersyndikat),
			_ => Err(()),
		}
	}
}

#[cfg(feature = "server")]
impl ToSql for Store {
	fn to_sql(&self) -> rusqlite::Result<ToSqlOutput<'_>> {
		match self {
			Store::Aldi => 0.to_sql(),
			Store::Edeka => 1.to_sql(),
			Store::Dm => 2.to_sql(),
			Store::Lidl => 3.to_sql(),
			Store::Rewe => 4.to_sql(),
			Store::Tegut => 5.to_sql(),
		}
	}
}

#[cfg(feature = "server")]
impl ToSql for Account {
	fn to_sql(&self) -> rusqlite::Result<ToSqlOutput<'_>> {
		match self {
			Account::Sumpf => 0.to_sql(),
			Account::Heinersyndikat => 1.to_sql(),
		}
	}
}
